/***************************************************************************
                          agent.h  -  description
                             -------------------
    begin                : Sat Feb 10 2001
    copyright            : (C) 2001 by Michael Ramsza
    email                : mramsz@sgh.waw.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* stuff related to agents' design */

 
/* definition of agent */
typedef struct Agent{
 int *wealth; /* cumulated payoffs across all markets */
 int prevmarket; /* previous market */
 int market; /* number of current market; 0, 1, ... */
 int **strs_set; /* pointer to set of strategies */
 int **v_points; /* pointer to virtual points */
 int *best; /* a list of best strategies for given market */
 int action[2]; /* list of used actions for local market and p_market */
 int lossnum; /* how many losses since the last market change */
 int tolerance; /* tolerance level  for loss number */
 int gr; /* group number - only if S_gr is set */
} AGENT;

/* only declaration here */
int hamming(int *, int *, int);

/* creating system of agents */
/* nom - number of markets
   noa - number of agents
   nos - number of strategies
   noh - number of different histories - HM
   a   - pointer to pointer to list of all agents
   */
int create_all_agents(int nom, 
                      int noa, 
								      int nos,
								      int noh,
								      AGENT **a,
											int invest)
{
    int i,j,l;    

    *a = (AGENT *) malloc(noa*sizeof(AGENT));
    for(i=0; i<noa; i++){    
	  (*a)[i].wealth = (int *) malloc((nom+1)*sizeof(int));
		for(j=0; j<=nom; j++)
				(*a)[i].wealth[j]=0;
	  (*a)[i].market=pf_uniformk(nom);
 	  (*a)[i].strs_set = (int **) malloc(nos*sizeof(int *));
	  for(j=0; j<nos; j++)
	      (*a)[i].strs_set[j] = (int *) malloc(noh*sizeof(int));
		/* force investments */
		if(!invest){
			for(j=0; j<nos; j++){
  	            for(l=0; l<noh; l++){
    	              (*a)[i].strs_set[j][l] = pf_uniform1(); }}}
		/* allow no investments */
		if(invest){
			for(j=0; j<nos; j++){
  	            for(l=0; l<noh; l++){
    	              (*a)[i].strs_set[j][l] = pf_uniformk(3)-1; }}}
	  (*a)[i].v_points = (int **) malloc((nom+1)*sizeof(int *));
	  for(j=0; j<=nom; j++)
	      (*a)[i].v_points[j] = (int *) malloc(nos*sizeof(int));
		for(j=0; j<=nom; j++)
			for(l=0; l<nos; l++) (*a)[i].v_points[j][l]=0;
	  (*a)[i].best = (int *) malloc((nom+1)*sizeof(int));
	  for(j=0; j<=nom; j++) 
	      (*a)[i].best[j] = pf_uniformk(nos);
	  (*a)[i].action[0] = pf_uniform1(); 
	  (*a)[i].action[1] = pf_uniform1();  }
    return 0;
}		

/* creating system of identical agents */
/* nom - number of markets
   noa - number of agents
   nos - number of strategies
   noh - number of different histories - HM
   a   - pointer to pointer to list of all agents
   */
int create_all_agents1(int nom,
                      int noa,
		      int nos,
		      int noh,
		      AGENT **a, int invest)
{
    int i,j,l;

    *a = (AGENT *) malloc(noa*sizeof(AGENT));
    for(i=0; i<noa; i++){
	  	(*a)[i].wealth = (int *) malloc((nom+1)*sizeof(int));
			for(j=0; j<=nom; j++)
					(*a)[i].wealth[j]=0;
	  	(*a)[i].market=pf_uniformk(nom);
 	  	(*a)[i].strs_set = (int **) malloc(nos*sizeof(int *));
	  	for(j=0; j<nos; j++)
	   	   (*a)[i].strs_set[j] = (int *) malloc(noh*sizeof(int));
	  	/* force investments */
		if(!invest){



			for(j=0; j<nos; j++){
  	            for(l=0; l<noh; l++){
    	              (*a)[i].strs_set[j][l] = pf_uniform1(); }}}
		/* allow no investments */
		if(invest){
			for(j=0; j<nos; j++){
  	            for(l=0; l<noh; l++){
    	              (*a)[i].strs_set[j][l] = pf_uniformk(3)-1; }}}
	  	(*a)[i].v_points = (int **) malloc((nom+1)*sizeof(int *));
	  	for(j=0; j<=nom; j++)
	   	   (*a)[i].v_points[j] = (int *) malloc(nos*sizeof(int));
			for(j=0; j<=nom; j++)
				for(l=0; l<nos; l++) (*a)[i].v_points[j][l]=0;
	  	(*a)[i].best = (int *) malloc((nom+1)*sizeof(int));
	  	for(j=0; j<=nom; j++)
	   	   (*a)[i].best[j] = 0;
	  	(*a)[i].action[0] = 1;
	  	(*a)[i].action[1] = 1;  }
    return 0;
}		

// This is third version of the program ...
// Now the loss tolerance is based on Hamming distance

int create_all_agents2(int nom,
                      int noa,
		      int nos,
		      int noh,
		      AGENT **a, int invest)
{
    int i,j,l;

    *a = (AGENT *) malloc(noa*sizeof(AGENT));
    for(i=0; i<noa; i++){
	  	(*a)[i].wealth = (int *) malloc((nom+1)*sizeof(int));
			for(j=0; j<=nom; j++)
					(*a)[i].wealth[j]=0;
	  	(*a)[i].market=pf_uniformk(nom);
 	  	(*a)[i].strs_set = (int **) malloc(nos*sizeof(int *));
	  	for(j=0; j<nos; j++)
	   	   (*a)[i].strs_set[j] = (int *) malloc(noh*sizeof(int));
	  	/* force investments */
		if(!invest){
			for(j=0; j<nos; j++){
  	            for(l=0; l<noh; l++){
    	              (*a)[i].strs_set[j][l] = pf_uniform1(); }}}
		/* allow no investments */
		if(invest){
			for(j=0; j<nos; j++){
  	            for(l=0; l<noh; l++){
    	              (*a)[i].strs_set[j][l] = pf_uniformk(3)-1; }}}
	  	(*a)[i].v_points = (int **) malloc((nom+1)*sizeof(int *));
	  	for(j=0; j<=nom; j++)
	   	   (*a)[i].v_points[j] = (int *) malloc(nos*sizeof(int));
			for(j=0; j<=nom; j++)
				for(l=0; l<nos; l++) (*a)[i].v_points[j][l]=0;
	  	(*a)[i].best = (int *) malloc((nom+1)*sizeof(int));
	  	for(j=0; j<=nom; j++)
	   	   (*a)[i].best[j] = 0;
	  	(*a)[i].action[0] = 1;
	  	(*a)[i].action[1] = 1;  
		// computing Hamming distance
    if(nos>1)
    {
  		(*a)[i].lossnum = 0; // no losses
	  	if(hamming((*a)[i].strs_set[0], (*a)[i].strs_set[1], noh)>noh/2){
		  	(*a)[i].tolerance= 50;// this is high tolerance
  		} else {
  			(*a)[i].tolerance=50; // this is low tolerance
  		}
    } else {
      (*a)[i].tolerance=8; // this is tolerance if one strategy only
    }

	}
    return 0;
}		


// This is fourth version of the program ...
// market switch is based on tolerance level
// works only with one strategy
// half of the agents have strategy 11..111
// the other will have anticoorelated strategy -1-1-1 ... -1
// Warning: this is ad hoc programming!

int create_all_agents3(int nom,
                      int noa,
		      int nos,
		      int noh,
		      AGENT **a, int invest)
{
    int i,j,l;
    int *str; // random strategy
    int *ac_str; // anticorrelated strategy

    printf("WARNING\ncreate_all_agents:: You are using overridden system of strategies. ");
    printf("The global variable G_Cooked is set!\n");

    /* generating random strategy and anticorrelated strategies */
    str = (int  *) malloc(noh*sizeof(int));
    ac_str = (int  *) malloc(noh*sizeof(int));
    for(l=0; l<noh; l++){
      str[l] = pf_uniform1();
      if(str[l]==1) ac_str[l]=-1;
      if(str[l]==-1) ac_str[l]=1;
    }

    *a = (AGENT *) malloc(noa*sizeof(AGENT));
    for(i=0; i<noa; i++){
      if (i%2 == 1) { (*a)[i].gr =1; } else { (*a)[i].gr=0; }
	  	(*a)[i].wealth = (int *) malloc((nom+1)*sizeof(int));
			for(j=0; j<=nom; j++)
					(*a)[i].wealth[j]=0;
	  	//probability of starting at first market!!!
      (*a)[i].market=pf_2point(0.2);
 	  	(*a)[i].strs_set = (int **) malloc(nos*sizeof(int *));
	  	for(j=0; j<nos; j++)
	   	   (*a)[i].strs_set[j] = (int *) malloc(noh*sizeof(int));

	  /* force investments */
    /* only this part works with G_Cooked */
			for(j=0; j<nos; j++){
  	            for(l=0; l<noh; l++){
    	              if(i%2 == 1)
                        { (*a)[i].strs_set[j][l] = str[l];} else {
                          (*a)[i].strs_set[j][l] = ac_str[l];} }}


	  	(*a)[i].v_points = (int **) malloc((nom+1)*sizeof(int *));
	  	for(j=0; j<=nom; j++)
	   	   (*a)[i].v_points[j] = (int *) malloc(nos*sizeof(int));
			for(j=0; j<=nom; j++)
				for(l=0; l<nos; l++) (*a)[i].v_points[j][l]=0;
	  	(*a)[i].best = (int *) malloc((nom+1)*sizeof(int));
	  	for(j=0; j<=nom; j++)
	   	   (*a)[i].best[j] = 0;
	  	(*a)[i].action[0] = 1;
	  	(*a)[i].action[1] = 1;
		// computing Hamming distance
    if(nos>1)
    {
  		(*a)[i].lossnum = 0; // no losses
	  	if(hamming((*a)[i].strs_set[0], (*a)[i].strs_set[1], noh)>noh/2){
		  	(*a)[i].tolerance= 50;// this is high tolerance
  		} else {
  			(*a)[i].tolerance=50; // this is low tolerance
  		}
    } else {
      (*a)[i].tolerance=pf_uniformk(25)+5; // random tolerance !!!!
    }

	}
    /* removing random and anticorrelated strategies */
    free(str);
    free(ac_str);
    return 0;
}		

/* a - pointer to list of agents
   noa - number of agents
   nom - number of markets
   h - pointer to set of all histories
   lenh - history length  */
int reply(AGENT *a, int noa, int nom, int **h, int lenh){
    int l, lm;
    for(l=0; l<noa; l++){
			lm=a[l].market;
			a[l].action[0] = a[l].strs_set[a[l].best[lm]][hist2num(h[lm],lenh)];
			a[l].action[1] = a[l].strs_set[a[l].best[nom]][hist2num(h[nom],lenh)];
    }
    return 0;
}

/* computes payoffs and updates welath levels */
/* a - pointer to list of all agents 
   noa - number of agents
   nom - number of markets
   v - pointer to vector of volumes
   p - pointer to vector of prices */
int wealth_update(AGENT *a, int noa, int nom, int *v, int *p){
    int l, lm;
    for(l=0; l<noa; l++){
			lm = a[l].market;
			a[l].wealth[lm] += - p[lm]*a[l].action[0];
			a[l].wealth[nom] += - p[nom]*a[l].action[1];
    }
    return 0;
}

/* computes payoffs and updates welath levels */
// updates numloss level -> see also market link update
/* a - pointer to list of all agents 
   noa - number of agents
   nom - number of markets
   v - pointer to vector of volumes
   p - pointer to vector of prices */
int wealth_update1(AGENT *a, int noa, int nom, int *v, int *p){
    int l, lm;
    for(l=0; l<noa; l++){
			lm = a[l].market;
			a[l].wealth[lm] += - p[lm]*a[l].action[0];
			a[l].wealth[nom] += - p[nom]*a[l].action[1];
			// increasing number of losses if strategy failed
			if(- p[lm]*a[l].action[0]<0){
				a[l].lossnum++;
			}
    }
    return 0;
}

/* updates virtual points for all agents and all markets 
   this is naive update */
/* a   - pointer to set of all agetnts
   noa - number of agents 
   nom - number of markets
   nos - number of strategies
   h - pointer to set of all histories
   hl - length of history
   p - pointer to list of prices
*/   
int vpoints_update(AGENT *a, 
                   int noa, 
		               int nom,
		               int nos,
             		   int **h,
             		   int hl,
            		   int *p){
    int q, w, e, bit3, act;
    for(q=0; q<noa; q++){
			for(w=0; w<=nom; w++){
	    	for(e=0; e<nos; e++){
					bit3 = hist2num(h[w],hl);
					act = a[q].strs_set[e][bit3];
					if(act*p[w]<0) a[q].v_points[w][e]++;
					if(act*p[w]>0) a[q].v_points[w][e]--;
					/* if zero then ... we can play with it */
	    	}
			}
    }
		/* updating best strategies for all markets */
    for(q=0; q<noa; q++){
			/* searching procedure for single agent */
			for(w=0; w<=nom; w++){
	    	bit3=a[q].v_points[w][0]; act=0;
	    	for(e=0; e<nos; e++){
					if(bit3<a[q].v_points[w][e]){
		    		act=e;
		    		bit3=a[q].v_points[w][e];
					}
	    		a[q].best[w]=act;
	    	}
			} /* end of searching procedure */
    }
    return 0;
}

/* updates virtual points for all agents and all markets
   it is supposed to be sophisticated update with random pick ;-)  */
/* a   - pointer to set of all agetnts
   noa - number of agents
   nom - number of markets
   nos - number of strategies
   h - pointer to set of all histories
   hl - length of history
   p - pointer to list of prices
*/
int vpoints_update1(AGENT *a,
                   int noa,
		   int nom,
		   int nos,
		   int **h,
		   int hl,
		   int *p){
    int q, w, e, bit3, act, brep;
    for(q=0; q<noa; q++){
	for(w=0; w<=nom; w++){
	    for(e=0; e<nos; e++){
		bit3 = hist2num(h[w],hl);
		act = a[q].strs_set[e][bit3];
		if(act*p[w]<0) a[q].v_points[w][e]++;
		if(act*p[w]>0) a[q].v_points[w][e]--;
		/* if zero then ... we can play with it */
	    	}
			}
    }
		/* updating best strategies for all markets */
    for(q=0; q<noa; q++){
			/* searching procedure for single agent */
			for(w=0; w<=nom; w++){
/*
	    	bit3=a[q].v_points[w][0]; act=0;
	    	for(e=0; e<nos; e++){
					if(bit3<a[q].v_points[w][e]){
		    		act=e;
		    		bit3=a[q].v_points[w][e];
					}
	    		a[q].best[w]=act;
	    	} */
        /* searching for max. value */
				bit3 = a[q].v_points[w][0];
				for(e=0; e<nos; e++){
					if(bit3<a[q].v_points[w][e]) bit3 = a[q].v_points[w][e];
				}
				/* computing number of first best repliers */
				brep =0;
				for(e=0; e<nos; e++){
					if(bit3==a[q].v_points[w][e]) brep++;
				}
				/* random pick */
				for(e=0; e<nos; e++){
					if(bit3==a[q].v_points[w][e]){
						if( pf_2point(1.0 / (float) brep)){ a[q].best[w] = e; e=nos; } else { brep--; }
					}
				}
			} /* end of searching procedure */
    }
    return 0;
}


/* resets market assignments */
/* a - pointer to list of all agents
   noa - number of agents
   nom - number of markets */
int marketlink_update(AGENT *a, int noa, int nom){
    int q, l, w, pos;
    for(q=0; q<noa; q++){
			a[q].prevmarket=a[q].market;
			pos=0; w=a[q].wealth[0];
				for(l=0; l<nom; l++){
	    		if(w<a[q].wealth[l]){
						pos=l; w = a[q].wealth[l];
	    		}
			}
			a[q].market=pos;
    }
    return 0;
}

/* resets market assignments with random choice between ties */
/* a - pointer to list of all agents
   noa - number of agents
   nom - number of markets */
int marketlink_update1(AGENT *a, int noa, int nom){
    int q, l, wmax, wmaxnum;
    for(q=0; q<noa; q++){
			a[q].prevmarket=a[q].market; /* storing last market */
			/* searching for max value */
			wmax=a[q].wealth[0];
			for(l=0; l<nom; l++)
				if(a[q].wealth[l]>wmax) wmax=a[q].wealth[l];
			/* computing number of first best replies */
			wmaxnum=0;
			for(l=0; l<nom; l++)
				if(wmax==a[q].wealth[l]) wmaxnum++;
			/* random pick */
			for(l=0; l<nom; l++){
					if(wmax==a[q].wealth[l]){
						if( pf_2point(1.0 / (float) wmaxnum)){ a[q].market = l; l=nom; } else { wmaxnum--; }
					}
				} /* end of searching procedure */
    }
    return 0;
}


// NEW VERSION NEW VERSION
/* resets market assignments */

// this is not random since we only look up a number of losses !!!
/* a - pointer to list of all agents
   noa - number of agents
   nom - number of markets WARNING: this version works only for
   2 markets, hence nom is not used - however it may be used for future projects
*/
int marketlink_update2(AGENT *a, int noa, int nom){
    int q;
    for(q=0; q<noa; q++){
			a[q].prevmarket=a[q].market;
			// checking tolerance level
			if(a[q].lossnum>= a[q].tolerance){
				// clearing nummloss
				a[q].lossnum=0;
				//switching markets
				if(a[q].market){
					a[q].market=0;
				} else {
						a[q].market=1;
				}
			} // if end
    } // for end
    return 0;
}

int remove_all_agents(AGENT *a, int noa, int nos, int nom){
    int l, k;
    for(l=0; l<noa; l++){
	free(a[l].wealth);
	free(a[l].best);
	for(k=0; k<nos; k++) free(a[l].strs_set[k]);
	free(a[l].strs_set);
	for(k=0; k<=nom; k++) free(a[l].v_points[k]);
	free(a[l].v_points);
    }
    free(a);
    return 0;
}

// computing Hamming distance
int hamming(int *sa, int *sb, int noh){
    int l, k;
  	k=0;
    for(l=0; l<noh; l++){
		k+=(sa[l]-sb[l])*(sa[l]-sb[l]);
    }
    k=k/4; // this is not correct ;)
    return k;
}

