
/***************************************************************************
                          msmg.c  -  description
                             -------------------
    begin                : Sat Feb 10 2001
    copyright            : (C) 2001 by Michael Ramsza
    email                : mramsz@sgh.waw.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* Local/global MG simulation */

#define DEBBUG0 1
#define DEBBUG1 1

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "random.h" /* generating pseudo-random numbers */
#include "configure.h" /* reading config file */
#include "history.h" /* management of histories */
#include "agent.h" /* definitions and stuff related to agents */
#include "market.h" /* all stuff related to markets */
#include "stat_mg.h" /* all statistics */
#include "verbose.h" /* stuff related to various messgs to stdout */

int main(int argc, char *argv[])
{
 int G_NumOfAgents; /* number of agents */
 int G_NumOfMarkets; /* number of agents */
 int G_NumOfSteps; /* number of steps */
 int G_NumOfRuns; /* number of runs for random */
 int G_NumOfStrategies; /* number of strategies */
 int G_HistoryLength; /* number of history bits in memory */
 int G_NoInvestments; /* allows to withdraw from a market */
 int S_Hamming; /* enables computing of Hamming distance */
 int S_Scaling; /* enables scaling of agents */
 int S_SigmaTrigger; /* offset for computing variance */
 int G_AutoName; /* autonaming of data files */
 int G_Cooked; /* this will create strategies ina preset way */
 char G_DataFile[MAX_STRING_LENGTH]; /* path to output data file */
 char G_AverFile[MAX_STRING_LENGTH]; /* path to output data file */
 int T_Debbug; /* enables debbug messages to stdout */
 int T_Verbose; /* enables various messages to stdout */
 int S_Price; /* enables output of prices to data file */
 int S_Volume; /* enables printing volumes to data file */
 int S_Switch; /* enables printing volumes to data file */
 int S_PrintStep; /* */
 int S_Volatility; /* enables computing of volatility */
 int S_MidVolatilityPrint; /* enables printing of mid-volatility */
 int S_Wealth; /* enables fprinting of local/global wealth */
 int S_Buyers; /* enables computing of number of buyers thru all markets */
 int S_PrintTrigger; /* sets offset for fprinting all triensient variables */

 
 int i, t, time; /* counters */
 int HM; /* number of different histories */
 int HM2; /* number of different histories for global market */
 float alfa2, alfa3;
 FILE *data_file; /* output file */
 FILE *aver_file; /* output file for averages */
 FILE *config_file; /* config file */ 
 int **history; /* list of all histories */
 int *volume; /* list of volumes of all local/global markets */
 int *buyers; /* number of buyers on each market */
 int *price; /* list of prices on local/global markets */
 float *volatility; /* list of average volatilities */
 float *totvol; /* total volatility */
 AGENT *agents; /* pointer to list of all agents */
 char tempname[MAX_STRING_LENGTH]; /* temp for autonaming feature */




/* proper usage */
if(argc != 2){
  printf("\n Usage:\n %s <config file>\n", argv[0]);
  exit(1); }
 /* openning config file */
if( ((config_file = fopen(argv[1],"r"))) == NULL){
    printf("Can't open config file. Exiting ...\n");
    exit(1); }
/* reading parameters from config file */
configure(config_file, "G_NumOfAgents", "int", (void *) &G_NumOfAgents);
configure(config_file, "G_NumOfMarkets", "int", (void *) &G_NumOfMarkets);
configure(config_file, "G_NumOfSteps", "int", (void *) &G_NumOfSteps);
configure(config_file, "G_NumOfRuns", "int", (void *) &G_NumOfRuns);
configure(config_file, "G_NumOfStrategies", "int", (void *) &G_NumOfStrategies);
configure(config_file, "G_HistoryLength", "int", (void *) &G_HistoryLength);
configure(config_file, "S_Hamming", "int", (void *) &S_Hamming);
configure(config_file, "S_Scaling", "int", (void *) &S_Scaling);
configure(config_file, "S_SigmaTrigger", "int", (void *) &S_SigmaTrigger);
configure(config_file, "G_Cooked", "int", (void *) &G_Cooked);
configure(config_file, "G_DataFile", "string", (void *) &G_DataFile);
configure(config_file, "G_AverFile", "string", (void *) &G_AverFile);
configure(config_file, "T_Debbug", "int", (void *) &T_Debbug);
configure(config_file, "T_Verbose", "int", (void *) &T_Verbose);
configure(config_file, "S_Price", "int", (void *) &S_Price);
configure(config_file, "S_Volume", "int", (void *) &S_Volume);
configure(config_file, "S_Switch", "int", (void *) &S_Switch);
configure(config_file, "S_Volatility", "int", (void *) &S_Volatility);
configure(config_file, "S_MidVolatilityPrint", "int", (void *) &S_MidVolatilityPrint);
configure(config_file, "S_PrintStep", "int", (void *) &S_PrintStep);
configure(config_file, "S_Wealth", "int", (void *) &S_Wealth);
configure(config_file, "S_PrintTrigger", "int", (void *) &S_PrintTrigger);
configure(config_file, "S_Buyers", "int", (void *) &S_Buyers);
configure(config_file, "G_NoInvestments", "int", (void *) &G_NoInvestments);
configure(config_file, "G_AutoName", "int", (void *) &G_AutoName);
/* closing config file */
fclose(config_file);
/* autonaming of files if set */
if( G_AutoName){
	sprintf(tempname, "_%d_%d_%d_%d.d", G_NumOfMarkets, G_NumOfStrategies, G_HistoryLength, G_NumOfAgents);
	strcat(G_DataFile, tempname);
	sprintf(tempname, "_%d_%d_%d_%d.ad", G_NumOfMarkets, G_NumOfStrategies, G_HistoryLength, G_NumOfAgents);
	strcat(G_AverFile, tempname);
}
/* opening output data file */
if( (data_file = fopen(G_DataFile, "w")) == NULL){
  printf("Can't open file. Exiting ...\n");
  exit(1); }

/* opening output data file for averages */
if( (aver_file = fopen(G_AverFile, "w")) == NULL){
  printf("Can't open file. Exiting ...\n");
  exit(1); }



/* sets seed for rand() function */
pf_srand(pf_make_seed());

/* number of runs  - mean over random fake histories */
create_totvolatility(G_NumOfMarkets, &totvol);
for(t=0; t<G_NumOfRuns; t++){

  /* number of different histories for local/global markets */
  HM = 3; for(i=0; i<G_HistoryLength-1; i++) HM *= 3;
  HM2 = 2; for(i=0; i<G_HistoryLength-1; i++) HM2 *=2;

	/* alphas for different scales */
	alfa2 = (float) HM2/(float) G_NumOfAgents;
	alfa3 = (float) HM/(float) G_NumOfAgents;



  create_history1(G_NumOfMarkets, G_HistoryLength, &history);
  if(!G_Cooked)
    {
      create_all_agents2(G_NumOfMarkets, G_NumOfAgents, G_NumOfStrategies, HM, &agents, G_NoInvestments);
    } else {
      //  this part only if G_Cooked set
      // forcing required settings !!!!!
      G_NumOfStrategies=1;
      G_NumOfMarkets=2;
      create_all_agents3(G_NumOfMarkets, G_NumOfAgents, G_NumOfStrategies, HM, &agents, G_NoInvestments);
    }
  create_vol_price(G_NumOfMarkets, &volume, &price);
	create_volatility(G_NumOfMarkets, &volatility);
	create_buyers(G_NumOfMarkets, &buyers);

    /* main loop */
    for(time=0; time<G_NumOfSteps; time++){
		S_addtime(data_file, S_Price+S_Volume+S_Switch+S_Wealth+S_Buyers, time, S_PrintStep, S_PrintTrigger);
		current_state(T_Verbose, t, G_NumOfRuns, time, G_NumOfSteps);	
    reply(agents, G_NumOfAgents, G_NumOfMarkets, history, G_HistoryLength);
    vol_price(agents, G_NumOfAgents, G_NumOfMarkets, &volume, &price);
		compute_buyers(agents, G_NumOfAgents, G_NumOfMarkets, &buyers);
		S_buyers(S_Buyers, buyers, G_NumOfMarkets, data_file, time, S_PrintStep, S_PrintTrigger);
		S_price(S_Price, price, G_NumOfMarkets, data_file, S_PrintStep, time, S_PrintTrigger);
		S_volume(S_Volume, volume, G_NumOfMarkets, data_file, time, S_PrintStep, S_PrintTrigger);
		S_volatility(S_Volatility, S_MidVolatilityPrint, S_Scaling, G_NumOfAgents, aver_file, price, volatility, G_NumOfMarkets, S_SigmaTrigger, time, G_NumOfSteps);
    wealth_update1(agents, G_NumOfAgents, G_NumOfMarkets, volume, price);
    vpoints_update1(agents, G_NumOfAgents, G_NumOfMarkets, G_NumOfStrategies, history, G_HistoryLength, price);
    marketlink_update2(agents, G_NumOfAgents, G_NumOfMarkets);
		S_wealth(S_Wealth, agents, G_NumOfAgents, G_NumOfMarkets, data_file, time, S_PrintStep, S_PrintTrigger);
		S_switch(S_Switch, agents, G_NumOfAgents,data_file, time, S_PrintStep, S_PrintTrigger);
    rollhistory(history, G_NumOfMarkets, G_HistoryLength, price);
    // a good place to compute actaul Hamming distance
    S_ham(1, agents, G_NumOfAgents, G_NumOfMarkets, HM, data_file, time, S_PrintStep, S_PrintTrigger);
    S_Gr(G_Cooked, agents, G_NumOfAgents,data_file, time, S_PrintStep, S_PrintTrigger, G_NumOfMarkets);
		S_endline(data_file, S_Price+S_Volume+S_Switch+S_Wealth+S_Buyers, time, S_PrintStep, S_PrintTrigger);
    } /* end of main loop */

	S_TotVolatility(S_Volatility, volatility, totvol, G_NumOfMarkets, t, G_NumOfRuns, aver_file, alfa2, alfa3, G_NumOfAgents, G_HistoryLength);
  remove_all_agents(agents, G_NumOfAgents, G_NumOfStrategies, G_NumOfMarkets);
  remove_vol_price(price, volume);
  remove_history(G_NumOfMarkets, &history);
	remove_volatility(volatility);
	remove_buyers(buyers);
} /* end of main loop for different runs */
remove_totvolatility(totvol);

/* closing data file */
fclose(data_file);
fclose(aver_file);
printf("\n");
return 0;
} /* end of main() */
