/******************************************************************************/
/*                                                                            */ 
/*  RANDOM - Generate 32 bit period-free random numbers.                      */  
/*                                                                            */  
/*    void pf_srand ( long iseed ) - Set the random seed                      */  
/*    long pf_rand () - Return a 32 bit random integer in range 0-259199      */  
/*    long pf_randmax () - Return the maximum number which pf_rand may return */  
/*                                                                            */  
/* Copyright (c) 1993 by Academic Press, Inc.                                 */  
/*                                                                            */  
/* All rights reserved.  Permission is hereby granted, until further notice,  */  
/* to make copies of this diskette, which are not for resale, provided these  */  
/* copies are made from this master diskette only, and provided that the      */  
/* following copyright notice appears on the diskette label:                  */  
/* (c) 1993 by Academic Press, Inc.                                           */  
/*                                                                            */  
/* Except as previously stated, no part of the computer program embodied in   */  
/* this diskette may be reproduced or transmitted in any form or by any means,*/  
/* electronic or mechanical, including input into storage in any information  */  
/* system for resale, without permission in writing from the publisher.       */  
/*                                                                            */  
/* Produced in the United States of America.                                  */  
/*                                                                            */  
/* ISBN 0-12-479041-0                                                         */  
/*                                                                            */  
/******************************************************************************/  


/* 2000.11.03 - Michael Ramsza: 
	I rewrote some parts of the code and added
        a simple function ``generating'' a seed calling time(). 
	Originally, one could not use this function since 
	the numbers were just too large. New function
	make_seed() returns numbers not greater then 9999.  */
	
/* 2000.11.05 - Michael Ramsza:
	I added some simple functions for getting uniform distribution over
	various sets; I also added general function for getting	uniform
	distribution over arbitrary finite set of arbitrary type. */
   
  
#include <time.h>
#include <math.h>
#include<stdio.h>


/* These constants are from Press et. al. "Numerical Recipes in C"  
   Do not tamper with them unless you are an expert  */

#define IM 714025L      
#define IA 1366L        
#define IC 150889L      
#define TABLE_LENGTH 97 /* Length of shuffle table  */
  

static long seed = 797 ;           /* Keep the current seed here  */
static long randout ;              /* Random output  */
static long table[TABLE_LENGTH] ;  /* Keep shuffle table here  */
static int table_initialized = 0 ; /* Has it been initialized?  */
  
/*   Set the random seed  */    
void pf_srand ( long iseed )  
{  
   seed = iseed ;          /* We keep the static seed here  */
   table_initialized = 0 ; /* Must also rebuild table with it! */  
}  
  
/*   Return the maximum random number  */  
long pf_randmax ()  
{  
   return IM - 1L ;  
}  
  
/*   This is the actual random number generator  */  
long pf_rand ()     /* Return the next random number  */
{  
   int i ;  
  
   if (! table_initialized) /* Initialize shuffle table before use  */
	{  
      table_initialized = 1 ;  	/* Flag to avoid more inits  */
      for (i=0 ; i<TABLE_LENGTH ; i++) /* Fill entire table  */
		{ 
         seed = (IA * seed + IC) % IM ;  /* Make a random number  */
         table[i] = seed ;               /* Put it in the table  */
         }  
      seed = (IA * seed + IC) % IM ;     /* One more random number for upcoming first use  */
      randout = seed ;                   
      }  
  
   i = (int) ((double) TABLE_LENGTH * (double) randout / (double) IM) ;  
   randout = table[i] ;            /* This output comes from table  */
   seed = (IA * seed + IC) % IM ;  /* Make new random number  */
   table[i] = seed ;               /* to replace used entry  */
   return randout ;                /* then return old entry  */
}  

/* Generates seed from calendar time */
long pf_make_seed(void)
{	
	return (long) time(NULL) % 100000;
}

/* uniform distribution over -/+ 1 */
int pf_uniform1(void)
{
	int r;
	r=1;
	if( (double) pf_rand() < (double) pf_randmax() * 0.5)	r=-1;
	return r;
}

/* uniform distribution over interval [0,1] */
float pf_uniform01(void)
{
	return (float) pf_rand() / (float) pf_randmax();
}

/*  returns 1 with probability pr and 0 with prob. 1-pr */
int pf_2point(float pr)
{
		int success = 0 ;
		if( pf_uniform01() <= pr) success=1;
    return success;
}

/* uniform distribution over 0 through k-1 */
int pf_uniformk( int k)
{
	return pf_rand() % k; /* I guess it needs improvement */
}

/* uniform distribution on a given finite set */
/* aset - pointer to the first element of a set
	len - number of elements
	elemsize - size of a single element as returned by sizeof() */
/*
void * pf_uniform_univ(void * aset, int len, size_t elemsize)
{
	return aset + pf_uniformk(len)*elemsize	;
}
*/

