/***************************************************************************
                          stat_mg.h  -  description
                             -------------------
    begin                : Sat Feb 10 2001
    copyright            : (C) 2001 by Michael Ramsza
    email                : mramsz@sgh.waw.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* various "statistical" measures of efficiency of local/global markets */

int create_buyers(int nom, int **v){
	int l;
	*v = (int *) malloc((nom+1)*sizeof(float));
	for(l=0; l<=nom; l++) (*v)[l]=0;
	return 0;
}

int remove_buyers(int *v){
	free(v);
	return 0;
}

int create_volatility(int nom, float **v){
	int l;
	*v = (float *) malloc((nom+1)*sizeof(float));
	for(l=0; l<=nom; l++) (*v)[l]=0;
	return 0;
}

int remove_volatility(float *v){
	free(v);
	return 0;
}


int create_totvolatility(int nom, float **v){
	int l;
	*v = (float *) malloc((nom+1)*sizeof(float));	
	for(l=0; l<=nom; l++) (*v)[l] = 0;
	return 0;
}

int remove_totvolatility(float *v){
	free(v);
	return 0;
}

int S_TotVolatility(int stat, float *v, float *tv, int nom, int r, int nor, FILE *df, float al2, float al3,
		int numagent, int histlen){
	int l;
	if(stat){
		for(l=0; l<=nom; l++) tv[l] += v[l];
		for(l=0; l<=nom; l++) v[l]=0;
		if(r+1 == nor){
			for(l=0; l<=nom; l++) tv[l] = tv[l]/ (float) nor;
			for(l=0; l<nom; l++) fprintf(df, "%16.3f ", tv[l]* (float) nom);
			fprintf(df, "%16.3f ", tv[nom]);
			fprintf(df, "    %3.4f    %3.4f", al2, al3);
			fprintf(df, "    %7d    %5d", numagent, histlen);
			fprintf(df," \n");
		}
	}
	return 0;
}


/* WARNING: it computes volatility on the market based
   on prices not actions played - if you change forming
   of prices it will also affect this function */

int S_volatility(int stat, /* S_Volatility */
                 int stat1, /* S_MidVolatilityPrint */
                 int statscal, /* S_Scaling */
                 int noa,
                 FILE *df, /* aver_file */
                 int *p, /* pointer to price list */
                 float *v, /* pointer to volatility list */
                 int nom,
                 int st, /* S_SigmaTrigger */
                 int ti, /* time */
                 int nostep){
	int l;
	float me;

  if(stat && (ti>st)){
		me = (float) ti-st;
		/* cleaning volatility list - not required ? */
		if(ti==st+1){
			for(l=0; l<=nom; l++) v[l]=0.0;
		}
		for(l=0; l<=nom; l++){
			v[l] = v[l]*(me-1.0);
			v[l] = (v[l] + (float) p[l] * (float) p[l])/me;
		}
		if((ti == nostep-1) && statscal){
				for(l=0; l<=nom; l++) v[l] = v[l]/(float) noa;
		}
		if((ti == nostep-1) && stat1){
			for(l=0; l<nom; l++) fprintf(df, "%16.3f ", v[l]* (float) nom);
			fprintf(df, "%16.3f ", v[nom]);
			fprintf(df,"\n");
		}
	}
	return 0;
}

int S_price(int stat, int *p, int nom, FILE *df, int step, int tt, int trig){
	int l;
	if(stat && ((tt%step)==0) && (tt > trig)){
		for(l=0; l<=nom; l++){
			fprintf(df, "%6d ", p[l]);
		}
/* 		fprintf(df, "\n"); */
	}
	return 0;
}

int S_volume(int stat, int *vol, int nom, FILE *df, int tt, int step, int trig){
	int l;
	if(stat && (tt%step == 0) && (tt>trig)){
		for(l=0; l<=nom; l++){
			fprintf(df,"%6d ", vol[l]);
		}
	}
	return 0;
}

int S_buyers(int stat, int *buy, int nom, FILE *df, int tt, int step, int trig){
	int l;
	if(stat && (tt%step == 0) && (tt>trig)){
		for(l=0; l<=nom; l++){
			fprintf(df,"%6d ", buy[l]);
		}
	}
	return 0;
}


int S_wealth(int stat, AGENT *a, int noa, int nom, FILE *df, int tt, int step, int trig){
	int l,q;
	double lw, gw;
	lw=0; gw=0;
	if(stat && (tt%step == 0) && (tt>trig)){
		for(l=0; l<noa; l++){
			gw += (double) a[l].wealth[nom];
			for(q=0; q<nom; q++){
				lw += (double) a[l].wealth[q];
			}
		}
		fprintf(df,"%16.3f %16.3f ", lw/(double) noa, gw/(double) noa);
	}
	return 0;
}

int S_switch(int stat, AGENT *a, int noa, FILE *df, int tt, int step, int trig){
	int l, sw;
	if(stat && (tt%step == 0) && (tt>trig)){
		sw=0;
		for(l=0; l<noa; l++){
			if(a[l].market != a[l].prevmarket) sw++;
		}
		fprintf(df, "%6d ", sw);
	}
	return 0;
}

int S_endline(FILE *df, int stat, int tt, int step, int trig){
	if((stat != 0) && (tt%step == 0) && (tt>trig)){
		fprintf(df,"\n");
	}
	return 0;
}

int S_addtime(FILE *df, int stat, int tm, int step, int trig){
	if((stat != 0) && (tm%step == 0) && (tm>trig)){
		fprintf(df,"%17d", tm);
	}
	return 0;
}

// computing Hamming distance min/aver/max
// it works only if NumOfStrategies == 1
int S_ham(int stat, AGENT *a, int noa, int nom, int noh, FILE *df, int tt, int step, int trig){
	int l,q, CM;
	double H, Hmin, Haver, Hmax, Naver;
	if(stat && (tt%step == 0) && (tt>trig)) // triggers for printing and computing
  {
    for(CM=0; CM<nom; CM++) // make CM<=nom to include global market
    {
      H=0.0; Hmin=1.0; Haver=0.0; Hmax=0.0; Naver=0.0;
      for(l=0; l<noa-1; l++)
      {
        if(a[l].market==CM)
        {
          for(q=l+1; q<noa; q++)
          {
            if(a[q].market==CM)
            {
              H = (double) hamming(a[l].strs_set[a[l].best[CM]], a[q].strs_set[a[q].best[CM]], noh); // computing Hamming distance between two used strategies
              H = H/((double) noh);
              if(H<Hmin) Hmin=H; // min
              if(H>Hmax) Hmax=H; // max
              Naver=Naver+1.0;
              Haver = (Haver*(Naver-1)+H)/Naver; // computing average Hamming distance
            }
          }
        }
      } // end for a single market
      fprintf(df,"%16.3f %16.3f %16.3f", Hmin, Haver, Hmax);//this is the last line for any market
    } // end for all markets
	} // end for all including triggers
	return 0;
}

// this will work only if G_Cooked is set
// now if you know what is G_Cooked than you know how this one works
int S_Gr(int stat, AGENT *a, int noa, FILE *df, int tt, int step, int trig, int nom){
	int l, sw, i;
	if(stat && (tt%step == 0) && (tt>trig)){
		for(i=0; i<nom; i++){
      sw=0;
	  	for(l=0; l<noa; l++){
		  	if((a[l].gr == 1) && (a[l].market==i)) sw++;
   		}
		  fprintf(df, "%6d ", sw);
    }
	}
	return 0;
}
